/***********************************************************************/
//#include "stdafx.h"
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "sprite.h"
/***********************************************************************/

/*------------------------------------------------------------------------*/
HBITMAP TSprite::hBmpSaveScreen = NULL;
int TSprite::nSprites = 0;
/*------------------------------------------------------------------------*/
void TSprite::SaveScreen( HDC hdc, HDC hdcMemory, int xScreen, int yScreen )
{
    // on sauve l'cran dans une bitmap pour des restaurations aises
    hBmpSaveScreen = CreateCompatibleBitmap( hdc, xScreen, yScreen );
    HBITMAP hbmpOld = (HBITMAP)::SelectObject(hdcMemory, hBmpSaveScreen );
    ::BitBlt(hdcMemory, 0, 0, xScreen, yScreen, hdc, 0, 0, SRCCOPY);
    ::SelectObject(hdcMemory, hbmpOld);
}
/*------------------------------------------------------------------------*/
void TSprite::RestScreen( HDC hdc, HDC hdcMemory, int xScreen, int yScreen )
{
    HBITMAP hbmpOld;
    hbmpOld = (HBITMAP)::SelectObject(hdcMemory, hBmpSaveScreen );
    ::BitBlt(hdc, 0, 0, xScreen, yScreen, hdcMemory, 0, 0, SRCCOPY );
    ::SelectObject(hdcMemory, hbmpOld);
}
/*------------------------------------------------------------------------*/
void TSprite::DestroySaveScreen()
{
    if( hBmpSaveScreen != NULL ) ::DeleteObject( hBmpSaveScreen );
}
/*------------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/


TSprite::TSprite( HINSTANCE hInst, char * aRes, int nbOrient, int nbCouche )
{
    if( hInst != NULL )
        Create( hInst, aRes, nbOrient, nbCouche );
}

/*------------------------------------------------------------------------*/
void TSprite::Create( HINSTANCE hInst, char * aRes, int nbOrient, int nbCouche )
{
    // Inits diverses
    hInstance = hInst;
    nOrient = nbOrient;
    nCouche =nbCouche;
    fPremier = TRUE;
    
    // Au dpart le sprite n'existe pas
    Set( 0, 0, 0, 0 );
    SetOld();
    Tue();
    
    // Tableau de dplacements
    aDeplace = new POINT[ nbOrient];
    
    // Bitmap de sauvegarde
    hBmpSave = NULL;
    
    // Chargement des images (LoadBitmap() est virtuelle)
    LoadBitmap( ahBmp, aRes );
    
    // Initialisation de la taille
    InitParam();
    
    // Incrementation de la static qui compte les sprites
    nSprites ++;
}

/*------------------------------------------------------------------------*/
TSprite::~TSprite()
{
    // Decrementation de la static qui compte les sprites 
    nSprites --;
    
    // DeleteObject est virtuelle
    if( ahBmp != NULL )     DeleteObject( ahBmp );
    if( hBmpSave != NULL )  ::DeleteObject( hBmpSave );
    if( aDeplace != NULL )  delete aDeplace;
    
    // Restauration memoire
    if( ! nSprites )    DestroySaveScreen();
}

/*------------------------------------------------------------------------*/
void TSprite::LoadBitmap( HBITMAP * & ahbitmap, char * aRes )
{
    char aResource[255];
    
    ahbitmap = new HBITMAP[nOrient * nCouche];
    for( int i = 0; i < nOrient; i ++ )
        for( int j = 0; j < nCouche; j ++ ) 
	{
            if( nOrient == 1 && nCouche == 1 )
                strcpy( aResource, aRes );
            else
                sprintf( aResource, "%s_%d_%d", aRes, i+1, j+1 );
            ahbitmap[i * nCouche + j ] = ::LoadBitmap( hInstance, aResource );
	}
}


/*------------------------------------------------------------------------*/
void TSprite::DeleteObject( HBITMAP * & ahbitmap )
{
    for( int i = 0; i < nOrient; i ++ )
    {
        for( int j = 0; j < nCouche; j ++ ) 
	{
            ::DeleteObject( ahbitmap[i * nCouche + j ] );
	}
    }
	
    delete [] ahbitmap;
    ahbitmap = NULL;
}

/*------------------------------------------------------------------------*/
void TSprite::InitParam()
{
    BITMAP bm;
    
    // Taille des images, pas de dplacement
    GetObject( ahBmp[0], sizeof(bm), (LPSTR)&bm );
    Taille_X = bm.bmWidth;
    Taille_Y = bm.bmHeight;
    BitColor = bm.bmPlanes * bm.bmBitsPixel;
    rDeplace = (Taille_X + Taille_Y) / 4;
    DWORD dwBytes = ((DWORD) bm.bmWidth * BitColor) / 32;
    if(((DWORD) bm.bmWidth * BitColor) % 32) {
	dwBytes++;
    }
    dwBytes *= 4;
    TailleImage = dwBytes * bm.bmHeight;
    
    // initialiser le tableau de dplacement
    for( int i = 0; i < nOrient; i ++ )
    {
	aDeplace[ i ].x = (long)(rDeplace * cos( 2 * PI * i / nOrient ));
	aDeplace[ i ].y = (long)(-rDeplace * sin( 2 * PI * i / nOrient ));
    }
}

/*------------------------------------------------------------------------*/
void TSprite::InitSprite( int xPos, int yPos, int nOrientation, int bEx )
{
    // Prise en compte de la nouvelle position
    Set( xPos, yPos, nOrientation, 0 );
    bVisible = bEx;
    // On doit indiquer qq chose
    bMove = TRUE;
    // Pas de restauration le premier coup
    bRestaure = FALSE;
}

/*------------------------------------------------------------------------*/
void TSprite::Avance( int Orientation )
{
    x += aDeplace[ Orientation ].x;
    y += aDeplace[ Orientation ].y;
    bMove = TRUE;
}

/*------------------------------------------------------------------------*/
void TSprite::Save( HDC /*hdc*/, HDC /*hdcMemory*/ )
{
/*
// initialiser une bitmap pour la sauvegarde
if( ! hBmpSave )
hBmpSave = CreateCompatibleBitmap( hdc, Taille_X, Taille_Y );
// Copier le fond dans la bitmap
HBITMAP hbmpOld = (HBITMAP)::SelectObject(hdcMemory, hBmpSave );
::BitBlt(hdcMemory, 0, 0, Taille_X, Taille_Y, hdc, x, y, SRCCOPY);
    ::SelectObject(hdcMemory, hbmpOld);*/
}

/*------------------------------------------------------------------------*/
void TSprite::Restaure( HDC hdc, HDC hdcMemory )
{
    if( bRestaure )
    {
	HBITMAP hbmpOld;
	hbmpOld = (HBITMAP)::SelectObject(hdcMemory, hBmpSaveScreen );
	::BitBlt(hdc, xOld, yOld, Taille_X, Taille_Y, hdcMemory, xOld, yOld, SRCCOPY );
	::SelectObject(hdcMemory, hbmpOld);
    }
    else
	bRestaure = TRUE;
}

/*------------------------------------------------------------------------*/
void TSprite::AfficheBlackSprite( HDC hdc, HDC hdcMemory, DWORD dwRop)
{
/*    if( fPremier )
{
return;
// Recup des infos pour la palette
GetDIBits(
hdc,           // handle to device context
ahBmp[0],      // handle to bitmap
NULL,   // first scan line to set in destination bitmap
NULL,   // number of scan lines to copy
NULL //lpvBits//,    // address of array for bitmap bits
&bitmapinfo, // address of structure with bitmap data
DIB_RGB_COLORS// RGB or palette index
);

  int palNumEntries = 256;
  LPLOGPALETTE pLogPal = (LPLOGPALETTE) new char[2 * sizeof(WORD) +
  palNumEntries * sizeof(PALETTEENTRY)];
  
    pLogPal->palVersion = 0x300;
    pLogPal->palNumEntries = palNumEntries;
    for(int i = 0; i < palNumEntries; i++) 
    {
    pLogPal->palPalEntry[i].peRed = bitmapinfo.bmiColors[i].rgbRed;
    pLogPal->palPalEntry[i].peGreen = bitmapinfo.bmiColors[i].rgbGreen;
    pLogPal->palPalEntry[i].peBlue = bitmapinfo.bmiColors[i].rgbBlue;
    pLogPal->palPalEntry[i].peFlags = 0;
    }
    hPalette = ::CreatePalette(pLogPal);
    delete pLogPal;
    
      fPremier = FALSE;
      
        }
    */ 
    
    Copy( ahBmp[Orient * nCouche + Couche], hdc, hdcMemory, x, y, dwRop );
    SetOld();
    bMove = FALSE;
}

/*------------------------------------------------------------------------*/
void TSprite::Copy( HBITMAP hBmp, HDC hdc, HDC hdcMemory, int xPos, int yPos, DWORD dwRop )
{
    HBITMAP hbmpOld;
    hbmpOld = (HBITMAP)::SelectObject(hdcMemory, hBmp );
    ::BitBlt(hdc, xPos, yPos, Taille_X, Taille_Y, hdcMemory, 0, 0, dwRop );
    ::SelectObject(hdcMemory, hbmpOld);
}
